package com.cusc.nirvana.user.rnr.fp.controller;

import com.cusc.nirvana.user.rnr.fp.dto.FpRnrSmsInfoDTO;
import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.fp.service.IFpRnrSmsInfoService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 实名业务短信信息(FpRnrSmsInfo)表控制层
 *
 * @author yuy336
 * @since 2022-04-19 20:39:07
 */
@RestController
@RequestMapping("/fpRnrSmsInfo")
public class FpRnrSmsInfoController {
    /**
     * 服务对象
     */
    @Autowired
    private IFpRnrSmsInfoService fpRnrSmsInfoService;

    /**
     * 分页查询
     *
     * @param bean 筛选条件
     * @return 查询结果
     */
    @PostMapping("/queryByPage")
    @ApiOperation(value = "分页查询", notes = "分页查询")
    public Response<PageResult<FpRnrSmsInfoDTO>> queryByPage(@RequestBody FpRnrSmsInfoDTO bean) {
        return Response.createSuccess(fpRnrSmsInfoService.queryByPage(bean));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
   @PostMapping("/getByUuid")
    public Response<FpRnrSmsInfoDTO> getByUuid(@RequestBody FpRnrSmsInfoDTO bean) {
        return Response.createSuccess(fpRnrSmsInfoService.getByUuid(bean));
    }
    
     /**
     * 通过主键查询单条数据
     *
     * @param bean 实体
     * @return 单条数据
     */
   @PostMapping("/queryByList")
    public Response<List<FpRnrSmsInfoDTO>> queryByList(@RequestBody FpRnrSmsInfoDTO bean) {
        return Response.createSuccess(fpRnrSmsInfoService.queryByList(bean));
    }

    /**
     * 新增数据
     *
     * @param bean 实体
     * @return 新增结果
     */
    @PostMapping("/add")
    public Response add(@RequestBody FpRnrSmsInfoDTO bean) {
       return Response.createSuccess(fpRnrSmsInfoService.insert(bean));
    }

    /**
     * 编辑数据
     *
     * @param bean 实体
     * @return 编辑结果
     */
    @PostMapping("/update")
    public Response update(@RequestBody FpRnrSmsInfoDTO bean) {
        return Response.createSuccess(fpRnrSmsInfoService.update(bean));
    }

    /**
     * 删除数据
     *
     * @param bean 实体
     * @return 删除是否成功
     */
    @PostMapping("/deleteById")
    public Response<Boolean> deleteById(@RequestBody FpRnrSmsInfoDTO bean) {
        return Response.createSuccess(fpRnrSmsInfoService.deleteById(bean));
    }

}

